class
	LIBPCRE_DEMO

inherit
	ARGUMENTS
	PCRE_CONSTANTS
	P_PCRE_COMPILE_STATIC
	P_PCRE_EXEC_STATIC

create
	make

feature {NONE}

	make
		local
			l_return: BOOLEAN
			l_pattern, l_subject: detachable STRING_8
			l_regex: detachable CE_POINTER [PD_REAL_PCRE8_OR_16]
			l_error_msg: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_error_offset: CE_ARRAY [INTEGER_32]
			l_match_count: INTEGER_32
			l_offsets: detachable CE_ARRAY [INTEGER_32]
			l_i: INTEGER_32
		do
			-- get the pattern and subject
			if argument_count = 2 then
				l_pattern := argument (1)
				l_subject := argument (2)
			else
				Io.put_string ("Usage: " + argument (0) + " pattern subject%N")
				l_return := True
			end

			-- compile the regular expression
			if not l_return then
				check
					attached l_pattern
				end

				create {CE_POINTER_ARRAY [INTEGER_8]} l_error_msg.make (<<1>>)
				create {CE_INTEGER_32_ARRAY} l_error_offset.make (<<1>>)
				l_regex := pcre_compile (
						ce_string (l_pattern),             -- the pattern
						0,                                 -- default options
						l_error_msg,                       -- for error message
						l_error_offset,                    -- for error offset
						create {CE_NATURAL_8_POINTER}.make -- use default character tables
					)
				if l_regex.is_void then
					Io.put_string ("PCRE compilation failed at offset " + l_error_offset.item.out + ": " + eif_string (l_error_msg.item) + "%N")
					l_return := True
				end
			end

			-- execute the regular expression (do the matching)
			if not l_return then
				check
					attached l_regex and
					attached l_subject
				end

				create {CE_INTEGER_32_ARRAY} l_offsets.make (<<30>>) -- should be a multiple of 3
				l_match_count := pcre_exec (
						l_regex,                                        -- the compiled pattern
						create {CE_CLASS_POINTER [PD_PCRE_EXTRA]}.make, -- no extra data - we didn't study the pattern
						ce_string (l_subject),                          -- the subject string
						l_subject.count,                                -- the length of the subject
						0,                                              -- start at offset 0 in the subject
						0,                                              -- default options
						l_offsets,                                      -- output vector for substring information
						l_offsets.count                                 -- number of elements in the output vector
					)
				if l_match_count < 0 then
					-- an error occured (see `PCRE_CONSTANTS')
					inspect l_match_count
					when Pcre_error_nomatch then
						Io.put_string ("No matches :-(%N")
					else
						Io.put_string ("Matching error: " + l_match_count.out + "%N")
					end
					l_return := True
				end
			end

			-- print the matches
			if not l_return then
				check
					attached l_offsets and
					attached l_subject
				end

				if l_match_count = 0 then
					Io.put_string ("%NWARNING: There was not enough space in the offset vector to save all captured substrings!%N%N")
					l_match_count := l_offsets.count // 3
				end
				Io.put_string ("Matched area: " + l_subject.substring (l_offsets[0] + 1, l_offsets[1]) + "%N")
				if l_match_count > 1 then
					Io.put_string ((l_match_count - 1).out + " captured substring(s):%N")
					from
						l_i := 1
					until
						l_i = l_match_count
					loop
						Io.put_string ("- " + l_i.out + ": " + l_subject.substring (l_offsets[2 * l_i] + 1, l_offsets[2 * l_i + 1]) + "%N")
						l_i := l_i + 1
					end
				end
			end

			-- release memory used for the compiled pattern
			if attached l_regex and then not l_regex.is_void then
				free (l_regex)
			end
		end

end
